# This file was auto-generated by Fern from our API Definition.

import typing

SpanSearchStreamRequestPublicType = typing.Union[
    typing.Literal["general", "tool", "llm", "guardrail"], typing.Any
]
